%
%
%  Normalize TimePoints Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="Normalize Time Points" icon="Matlab" tooltip="Normalizes the voxel intensities for each time point of a 4D dataset.">
%          <Command>MatlabXT::XTNormalizeTimePoints(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
%  
%
%  Description:
%   
%   Normalizes the voxel intensities for each time point of a 4D dataset
%       (same mean and stddev).
%
%

function XTNormalizeTimePoints(aImarisApplicationID, aExcludeZero)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
    vImarisApplication = aImarisApplicationID;
end

% get the data set 
vDataSet = vImarisApplication.GetDataSet.Clone;
vNumberOfTimePoints = vDataSet.GetSizeT;

if vNumberOfTimePoints == 1
    msgbox('There is only one time point', 'Info');
    return;
end

vThreshold = 0;
if nargin == 1 
   vAnswer = inputdlg({'Exclude Voxels less than'}, 'Normalize TimePoints', 1, {'1'});
   if ~isempty(vAnswer)
       vThreshold = str2double(vAnswer(1)); 
   else 
       return;
   end

elseif nargin == 2
    if aExcludeZero == true
        vThreshold = 1;
    end
end

vImarisApplication.DataSetPushUndo('Normalize TimePoints');

vNumberOfChannels = vDataSet.GetSizeC;
vNumberOfChannelsSelected = 0;
for vChannelIndex = 1:vNumberOfChannels
    if vImarisApplication.GetChannelVisibility(vChannelIndex-1) == true
        vNumberOfChannelsSelected = vNumberOfChannelsSelected + 1;
    end
end

vSize = [vDataSet.GetSizeX,vDataSet.GetSizeY,vDataSet.GetSizeZ];

vProgressTotalCount = 2*vNumberOfChannelsSelected*vNumberOfTimePoints;
vProgressDisplay = waitbar(0,'Normalize TimePoints');
vProgressCount = 0;

for vChannelIndex = 1:vNumberOfChannels
    if vImarisApplication.GetChannelVisibility(vChannelIndex-1) == false
        continue;
    end
    
    % get total mean and variance
    vCount = 0;
    vSum = 0;
    vSumOfSquares = 0;
    for vTimeIndex = 1:vNumberOfTimePoints
        vImage = double(GetImage(vDataSet, vChannelIndex-1, vTimeIndex-1));        
        vDataVector = vImage(vImage>=vThreshold);
        vCountVolume = numel(vDataVector);
        if vCountVolume > 0
            vCount= vCount + vCountVolume;
            vSum = vSum + sum(vDataVector);
            vSumOfSquares = vSumOfSquares + sum(vDataVector.^2);
        end
        vProgressCount = vProgressCount + 1;
        waitbar(vProgressCount/vProgressTotalCount)
    end
    vMean = vSum/vCount;
    vVariance = vSumOfSquares/vCount - vMean*vMean;

    for vTimeIndex = 1:vNumberOfTimePoints
        vImage = double(GetImage(vDataSet, vChannelIndex-1, vTimeIndex-1));        
        vDataVector = vImage(vImage>=vThreshold);
        vCount = numel(vDataVector);
        if vCount > 0
            vMeanVolume = mean(vDataVector);
            vVarianceVolume = var(vDataVector);
            if vVarianceVolume == 0
                vVarianceVolume = vVariance;
            end
            vImage(vImage>=vThreshold) = vMean + ...
              (vDataVector-vMeanVolume)*sqrt(vVariance/vVarianceVolume);
            SetImage(vImage, vDataSet, vChannelIndex-1, vTimeIndex-1);
        end
        vProgressCount = vProgressCount + 1;
        waitbar(vProgressCount/vProgressTotalCount)
    end    
end
close(vProgressDisplay);

vImarisApplication.SetDataSet(vDataSet);


function aImage = GetImage(aDataSet, aChannel, aTime)
vSize = [aDataSet.GetSizeX, aDataSet.GetSizeY, aDataSet.GetSizeZ];
if strcmp(aDataSet.GetType, 'eTypeUInt8')
    aImage = zeros(vSize(1)*vSize(2), vSize(3), 'uint8');
    for vSlice = 1:vSize(3)
      aImage(:, vSlice) = typecast(aDataSet.GetDataSubVolumeAs1DArrayBytes(...
        0, 0, vSlice-1, aChannel, aTime, vSize(1), vSize(2), 1), 'uint8');
    end
elseif strcmp(aDataSet.GetType, 'eTypeUInt16')
    aImage = zeros(vSize(1)*vSize(2), vSize(3), 'uint16');
    for vSlice = 1:vSize(3)
      aImage(:, vSlice) = typecast(aDataSet.GetDataSubVolumeAs1DArrayShorts(...
        0, 0, vSlice-1, aChannel, aTime, vSize(1), vSize(2), 1), 'uint16');
    end
elseif strcmp(aDataSet.GetType, 'eTypeFloat')
    aImage = zeros(vSize(1)*vSize(2), vSize(3), 'single');
    for vSlice = 1:vSize(3)
      aImage(:, vSlice) = aDataSet.GetDataSubVolumeAs1DArrayFloats(...
        0, 0, vSlice-1, aChannel, aTime, vSize(1), vSize(2), 1);
    end
end
aImage = reshape(aImage, vSize);


function SetImage(aImage, aDataSet, aChannel, aTime)
vSize = [aDataSet.GetSizeX, aDataSet.GetSizeY, aDataSet.GetSizeZ];
if strcmp(aDataSet.GetType, 'eTypeUInt8')
    for vSlice = 1:vSize(3)
      aDataSet.SetDataSliceBytes(uint8(aImage(:, :, vSlice)), ...
        vSlice-1, aChannel, aTime);
    end
elseif strcmp(aDataSet.GetType, 'eTypeUInt16')
    for vSlice = 1:vSize(3)
      aDataSet.SetDataSliceShorts(uint16(aImage(:, :, vSlice)), ...
        vSlice-1, aChannel, aTime);
    end
elseif strcmp(aDataSet.GetType, 'eTypeFloat')
    for vSlice = 1:vSize(3)
      aDataSet.SetDataSliceFloats(single(aImage(:, :, vSlice)), ...
        vSlice-1, aChannel, aTime);
    end
end
